'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._before = undefined;

var _validationHelpers = require('./validationHelpers');

var _helpers = require('./helpers');

/**
 * Before function decorator
 *
 * @method _before
 * @param { function }
 *
 * @return { function }  decorator function
 */
/**
* @before decorator
*
* @author  Avraam Mavridis      <avr.mav@gmail.com>
*
*/
var _before = exports._before = function _before(beforeFunc) {
  if (!(0, _validationHelpers._isFunction)(beforeFunc)) {
    throw Error('a function should be passed to the @before decorator');
  }
  return function (key, target, descriptor) {
    var func = descriptor.value;
    (0, _helpers.descriptorIsFunc)(key, func);
    descriptor.value = function () {
      var beforeFuncRes = beforeFunc();

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var res = func.apply(this, args);
      if ((0, _validationHelpers._isPromise)(beforeFuncRes)) {
        return beforeFuncRes.then(function () {
          return res;
        });
      } else {
        return res;
      }
    };
    return descriptor;
  };
};