'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._trycatch = undefined;

var _helpers = require('./helpers');

var _validationHelpers = require('./validationHelpers');

/**
 * Try-catch decorator
 *
 * @method _timeout
 *
 * @param  { func } errorHandler
 *
 */
/**
* Try catch Decorator
*
* @author  Avraam Mavridis      <avr.mav@gmail.com>
*
*/
var _trycatch = exports._trycatch = function _trycatch(errorHandler) {

  if (!(0, _validationHelpers._isFunction)(errorHandler)) {
    throw Error('The ErrorHandler should be a function. ' + JSON.stringify(errorHandler) + ' is not a function');
  }

  return function (key, target, descriptor) {
    var func = descriptor.value;
    (0, _helpers.descriptorIsFunc)(key, func);
    descriptor.value = function () {
      var res = void 0;
      try {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        res = func.apply(this, args);
      } catch (e) {
        errorHandler(e);
      }
      return res;
    };
    return descriptor;
  };
};